"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.processCommands = exports.Device = void 0;
const tplink_smarthome_crypto_1 = require("tplink-smarthome-crypto");
const utils_1 = require("./utils");
Object.defineProperty(exports, "processCommands", { enumerable: true, get: function () { return utils_1.processCommands; } });
const device_networking_1 = __importDefault(require("./device-networking"));
class Device {
    static models = [
        'hs100',
        'hs105',
        'hs110',
        'hs110v2',
        'hs200',
        'hs220',
        'hs300',
        'lb100',
        'lb120',
        'lb130',
        'kl430',
    ];
    api;
    data;
    deviceNetworking;
    endSocketAfterResponse;
    model;
    unreliablePercent = 0;
    #deviceInfo;
    constructor({ model, port = 0, address = '0.0.0.0', alias, responseDelay = 0, unreliablePercent = 0, data = {}, }) {
        this.deviceNetworking = new device_networking_1.default({
            device: this,
            model,
            port,
            address,
            responseDelay,
        });
        this.model = model;
        const buildableData = { ...data };
        buildableData.model = model;
        if (alias != null)
            buildableData.alias = alias;
        // eslint-disable-next-line global-require, import/no-dynamic-require, @typescript-eslint/no-var-requires
        const SpecificDevice = require(`./devices/${model}`);
        this.#deviceInfo = new SpecificDevice(buildableData);
        this.#deviceInfo.initDefaults();
        this.api = this.#deviceInfo.api;
        this.data = this.#deviceInfo.data;
        this.unreliablePercent = unreliablePercent;
    }
    get address() {
        return this.deviceNetworking.address;
    }
    get port() {
        return this.deviceNetworking.port;
    }
    async start() {
        return this.deviceNetworking.start();
    }
    async stop() {
        return this.deviceNetworking.stop();
    }
    get children() {
        return this.#deviceInfo.children;
    }
    get mac() {
        return this.#deviceInfo.mac;
    }
    processMessage(msg, encryptFn, customizerFn) {
        this.#deviceInfo.reset();
        let responseUnencrypted = (0, utils_1.processCommands)(msg, this.api, this.#deviceInfo.constructor.errors, customizerFn);
        let response = encryptFn(responseUnencrypted);
        const badData = (0, utils_1.unreliableData)(this.unreliablePercent);
        if (badData !== undefined) {
            responseUnencrypted = badData;
            response = badData;
        }
        return { response, responseForLog: responseUnencrypted };
    }
    processUdpMessage(msg) {
        return this.processMessage(msg, tplink_smarthome_crypto_1.encrypt, (moduleName, methodName, methodResponse) => {
            // UDP only returns last two characters of child.id
            if (moduleName === 'system' &&
                methodName === 'get_sysinfo' &&
                methodResponse &&
                methodResponse.children &&
                methodResponse.children.length > 0) {
                methodResponse.children.forEach((child) => {
                    // eslint-disable-next-line no-param-reassign
                    child.id = child.id.slice(-2);
                });
            }
            return methodResponse;
        });
    }
    processTcpMessage(msg) {
        return this.processMessage(msg, tplink_smarthome_crypto_1.encryptWithHeader);
    }
}
exports.Device = Device;
//# sourceMappingURL=device.js.map