"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const node_dgram_1 = __importDefault(require("node:dgram"));
const node_events_1 = __importDefault(require("node:events"));
const debug_1 = __importDefault(require("debug"));
const log = (0, debug_1.default)('tplink-simulator:udp-server');
const logErr = (0, debug_1.default)('tplink-simulator:udp-server:error');
class UdpServer extends node_events_1.default {
    socketBound = false;
    socket = undefined;
    constructor() {
        super();
        this.setMaxListeners(25);
    }
    start({ port = 9999 } = {}) {
        return new Promise((resolve, reject) => {
            try {
                const socket = node_dgram_1.default.createSocket({ type: 'udp4', reuseAddr: true });
                this.socket = socket;
                socket.on('listening', () => {
                    const address = socket.address();
                    this.socketBound = true;
                    log('UDP server listening', address);
                    resolve(this);
                });
                socket.on('message', (msg, rinfo) => {
                    this.emit('message', msg, rinfo);
                });
                socket.on('error', (exception) => {
                    logErr(exception);
                    socket.close();
                    this.socketBound = false;
                    reject(exception);
                });
                socket.bind(port);
            }
            catch (err) {
                reject(err);
            }
        });
    }
    stop() {
        if (this.socketBound) {
            if (this.socket != null)
                this.socket.close();
            this.socketBound = false;
        }
    }
}
exports.default = new UdpServer();
//# sourceMappingURL=udp-server.js.map